#!/usr/bin/ruby
#
# You may specify the path to the FastCGI crash log (a log of unhandled
# exceptions which forced the FastCGI instance to exit, great for debugging)
# and the number of requests to process before running garbage collection.
#
# By default, the FastCGI crash log is RAILS_ROOT/log/fastcgi.crash.log
# and the GC period is nil (turned off).  A reasonable number of requests
# could range from 10-100 depending on the memory footprint of your app.
#
# Example:
#   # Default log path, normal GC behavior.
#   RailsFCGIHandler.process!
#
#   # Default log path, 50 requests between GC.
#   RailsFCGIHandler.process! nil, 50
#
#   # Custom log path, normal GC behavior.
#   RailsFCGIHandler.process! '/var/log/myapp_fcgi_crash.log'
#

# get the vendor directory into the load path well in advance, so
# we can pull rubygems from there
$LOAD_PATH.unshift(File.dirname(__FILE__) + "/../vendor")

# Workaround for bnc#579874 (ruby: String#unpack("V") does not work correctly on s390x)
if [1].pack('V').unpack('V')[0] != 1
  class String
    alias_method :broken_unpack, :unpack
    def unpack(spec)
      # use regular unpack first...
      unpacked = broken_unpack(spec)
      if spec.include? 'V'
        # ...then, if there's a 'V' anywhere, expand the spec
        # (need this to handle numeric and '*' parts of spec)...
        parts = spec.scan(/[0-9]+|[^0-9]/)
        spec = ''
        final = ''
        parts.each_with_index do |x,i|
          n = x.to_i
          if n > 0
            (n-1).times { spec += parts[i-1] }
          else
            spec += x
            final = x unless x == '*'
          end
        end
        # ...and match the expanded spec against the unpacked array...
        unpacked = unpacked.each_with_index.map do |x,i|
          # cope with '*' at end of spec
          format = (i >= spec.length || spec[i] == ?*) ? final[0] : spec[i]
          # if we've hit a 'V', shift it right 32 bits to get the correct value
          x >>= 32 if format == ?V
          x
        end
      end
      unpacked
    end
  end
end

require File.dirname(__FILE__) + "/../config/environment"
require 'fcgi_handler'

RailsFCGIHandler.process!
